#include <stdio.h>
#include <stdlib.h>

/* Number of jobs in the hold queue */
#define JOBQ     5

/* Process Control Block */
struct Job {
        int     pid;
        int     t;
        int     cpucycle;
        char    run[256];
        int     active;
};

int
main(int argc, char **argv)
{
        /*
         * Allocate contiguous block of memory for job queue that contains
         * pointers to each PCB, and initialise a pointer to the queue.
         */
        struct Job      ***p;
        struct Job       **queue;
        int                ct;
        int                ready;
        int                time;
        
        queue = calloc(JOBQ, sizeof(struct Job *));
        p = &queue;
        ct = 0;
        ready = 1;
        time = 0;
        
        /* Allocate memory for each PCB and initialise each block to zero */
        for (int i = 0; i < JOBQ; ++i) {
                queue[i] = calloc(1, sizeof(struct Job));
        }

        /* Load data into each PCB */
        queue[0]->pid = 0;
        queue[0]->t = 0;
        queue[0]->cpucycle = 10;

        queue[1]->pid = 1;
        queue[1]->t = 1;
        queue[1]->cpucycle = 2;
        
        queue[2]->pid = 2;
        queue[2]->t = 2;
        queue[2]->cpucycle = 3;
        
        queue[3]->pid = 3;
        queue[3]->t = 3;
        queue[3]->cpucycle = 1;
        
        queue[4]->pid = 4;
        queue[4]->t = 4;
        queue[4]->cpucycle = 5;
        
        /*
         * While jobs are ready, iterate over queue, if job has arrived and
         * has cpu cycles: set active bit; while job has cycles, increment
         * job run counter (#); and iterate over job queue, incrementing each
         * waiting job's wait counter (-). Increment time, and job status
         * pointer each cpu cycle. When job is finished, unset active bit.
         * Continue while unfinished jobs remain in queue. When all jobs
         * complete, display to screen job progression through queue.
         */
        do {
                for (int i = 0; i < JOBQ; ++i) {
                        if ((*p)[i]->t <= time && (*p)[i]->cpucycle > 0) {
                                (*p)[i]->active = 1;
                                while ((*p)[i]->cpucycle && (*p)[i]->active) {
                                        for (int x = 0; x < JOBQ; ++x) {
                                                if ((*p)[x]->active)
                                                        (*p)[x]->run[ct] = '#';
                                                else
                                                        (*p)[x]->run[ct] = '-';
                                        }
                                        ++ct;
                                        ++time;
                                        (*p)[i]->cpucycle--;
                                }
                                (*p)[i]->active = 0;
                                ready = 0;
                        }
                        for (int i = 0; i < JOBQ; ++i)
                                if ((*p)[i]->cpucycle > 0)
                                        ready = 1;
                }
        } while (ready == 1);

        for (int i = 0; i < JOBQ; ++i)
                printf("%s\n", (*p)[i]->run);
}